// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_gamma_correction.gen.h
 *
 * GENERATED on 2025 03 09 16:10:03 from gp_gamma_correction.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_GAMMA_CORRECTION_GEN_H
#define GP_GAMMA_CORRECTION_GEN_H

/**
 * @file gp_gamma_correction.gen.h
 * @brief Gamma corrections.
 */
/*
 * Copyright (C) 2012-2024 Cyril Hrubis <metan@ucw.cz>
 */

/**
 * @brief A macro to compose a RGB101010 pixel from linearized channel values.
 *
 * @param R A RGB101010 linearized R channel value.
 * @param G A RGB101010 linearized G channel value.
 * @param B A RGB101010 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB101010 pixel value.
 */
#define GP_PIXEL_CREATE_RGB101010_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 20) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 10) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB101010 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB101010_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB101010(p)] : \
	            GP_PIXEL_GET_R_RGB101010(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB101010 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB101010_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB101010 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB101010_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 4095 : 1023)

/**
 * @brief Returns a lookup table to encode RGB101010 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB101010_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u16 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB101010 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB101010_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB101010(p)] : \
	            GP_PIXEL_GET_G_RGB101010(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB101010 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB101010_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB101010 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB101010_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 4095 : 1023)

/**
 * @brief Returns a lookup table to encode RGB101010 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB101010_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u16 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB101010 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB101010_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB101010(p)] : \
	            GP_PIXEL_GET_B_RGB101010(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB101010 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB101010_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB101010 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB101010_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 4095 : 1023)

/**
 * @brief Returns a lookup table to encode RGB101010 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB101010_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u16 : NULL)

/**
 * @brief A macro to compose a xRGB8888 pixel from linearized channel values.
 *
 * @param R A xRGB8888 linearized R channel value.
 * @param G A xRGB8888 linearized G channel value.
 * @param B A xRGB8888 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A xRGB8888 pixel value.
 */
#define GP_PIXEL_CREATE_xRGB8888_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 16) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 8) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for xRGB8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_xRGB8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_xRGB8888(p)] : \
	            GP_PIXEL_GET_R_xRGB8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize xRGB8888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_xRGB8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized xRGB8888 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_xRGB8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode xRGB8888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_xRGB8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for xRGB8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_xRGB8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_xRGB8888(p)] : \
	            GP_PIXEL_GET_G_xRGB8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize xRGB8888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_xRGB8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized xRGB8888 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_xRGB8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode xRGB8888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_xRGB8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for xRGB8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_xRGB8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_xRGB8888(p)] : \
	            GP_PIXEL_GET_B_xRGB8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize xRGB8888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_xRGB8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized xRGB8888 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_xRGB8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode xRGB8888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_xRGB8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a RGBA8888 pixel from linearized channel values.
 *
 * @param R A RGBA8888 linearized R channel value.
 * @param G A RGBA8888 linearized G channel value.
 * @param B A RGBA8888 linearized B channel value.
 * @param A A RGBA8888 linearized A channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param A_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGBA8888 pixel value.
 */
#define GP_PIXEL_CREATE_RGBA8888_ENC(R, G, B, A, R_gamma_enc, G_gamma_enc, B_gamma_enc, A_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 24) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 16) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 8) \
	+ ((A_gamma_enc ? A_gamma_enc[(A)] : A) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGBA8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGBA8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGBA8888(p)] : \
	            GP_PIXEL_GET_R_RGBA8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGBA8888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGBA8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGBA8888 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGBA8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGBA8888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGBA8888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGBA8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGBA8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGBA8888(p)] : \
	            GP_PIXEL_GET_G_RGBA8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGBA8888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGBA8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGBA8888 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGBA8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGBA8888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGBA8888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGBA8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGBA8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGBA8888(p)] : \
	            GP_PIXEL_GET_B_RGBA8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGBA8888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGBA8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGBA8888 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGBA8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGBA8888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGBA8888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief Macro to get linearized A channel value for RGBA8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_A_RGBA8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_A_RGBA8888(p)] : \
	            GP_PIXEL_GET_A_RGBA8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGBA8888 pixel A channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGBA8888_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[3]) ? (pixmap)->gamma->lin[3]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGBA8888 pixel A channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGBA8888_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[3]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode RGBA8888 pixel A channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGBA8888_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[3]) ? (pixmap)->gamma->enc[3]->u8 : NULL)

/**
 * @brief A macro to compose a RGB888 pixel from linearized channel values.
 *
 * @param R A RGB888 linearized R channel value.
 * @param G A RGB888 linearized G channel value.
 * @param B A RGB888 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB888 pixel value.
 */
#define GP_PIXEL_CREATE_RGB888_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 16) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 8) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB888(p)] : \
	            GP_PIXEL_GET_R_RGB888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB888 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGB888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB888(p)] : \
	            GP_PIXEL_GET_G_RGB888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB888 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGB888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB888(p)] : \
	            GP_PIXEL_GET_B_RGB888(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB888 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode RGB888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a BGR888 pixel from linearized channel values.
 *
 * @param B A BGR888 linearized B channel value.
 * @param G A BGR888 linearized G channel value.
 * @param R A BGR888 linearized R channel value.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A BGR888 pixel value.
 */
#define GP_PIXEL_CREATE_BGR888_ENC(B, G, R, B_gamma_enc, G_gamma_enc, R_gamma_enc) \
	(0\
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 16) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 8) \
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 0) \
	)
/**
 * @brief Macro to get linearized B channel value for BGR888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_BGR888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_BGR888(p)] : \
	            GP_PIXEL_GET_B_BGR888(p)              \
)

/**
 * @brief Returns a lookup table to linearize BGR888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_BGR888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized BGR888 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_BGR888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode BGR888 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_BGR888_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for BGR888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_BGR888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_BGR888(p)] : \
	            GP_PIXEL_GET_G_BGR888(p)              \
)

/**
 * @brief Returns a lookup table to linearize BGR888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_BGR888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized BGR888 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_BGR888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode BGR888 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_BGR888_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized R channel value for BGR888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_BGR888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_BGR888(p)] : \
	            GP_PIXEL_GET_R_BGR888(p)              \
)

/**
 * @brief Returns a lookup table to linearize BGR888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_BGR888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized BGR888 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_BGR888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode BGR888 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_BGR888_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a RGB555 pixel from linearized channel values.
 *
 * @param R A RGB555 linearized R channel value.
 * @param G A RGB555 linearized G channel value.
 * @param B A RGB555 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB555 pixel value.
 */
#define GP_PIXEL_CREATE_RGB555_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 10) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 5) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB555 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB555_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB555(p)] : \
	            GP_PIXEL_GET_R_RGB555(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB555 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB555_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB555 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB555_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 127 : 31)

/**
 * @brief Returns a lookup table to encode RGB555 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB555_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB555 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB555_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB555(p)] : \
	            GP_PIXEL_GET_G_RGB555(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB555 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB555_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB555 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB555_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 127 : 31)

/**
 * @brief Returns a lookup table to encode RGB555 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB555_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB555 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB555_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB555(p)] : \
	            GP_PIXEL_GET_B_RGB555(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB555 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB555_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB555 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB555_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 127 : 31)

/**
 * @brief Returns a lookup table to encode RGB555 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB555_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a RGB565 pixel from linearized channel values.
 *
 * @param R A RGB565 linearized R channel value.
 * @param G A RGB565 linearized G channel value.
 * @param B A RGB565 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB565 pixel value.
 */
#define GP_PIXEL_CREATE_RGB565_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 11) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 5) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB565 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB565_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB565(p)] : \
	            GP_PIXEL_GET_R_RGB565(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB565 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB565_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB565 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB565_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 127 : 31)

/**
 * @brief Returns a lookup table to encode RGB565 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB565_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB565 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB565_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB565(p)] : \
	            GP_PIXEL_GET_G_RGB565(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB565 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB565_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB565 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB565_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 255 : 63)

/**
 * @brief Returns a lookup table to encode RGB565 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB565_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB565 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB565_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB565(p)] : \
	            GP_PIXEL_GET_B_RGB565(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB565 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB565_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB565 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB565_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 127 : 31)

/**
 * @brief Returns a lookup table to encode RGB565 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB565_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a RGB666 pixel from linearized channel values.
 *
 * @param R A RGB666 linearized R channel value.
 * @param G A RGB666 linearized G channel value.
 * @param B A RGB666 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB666 pixel value.
 */
#define GP_PIXEL_CREATE_RGB666_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 12) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 6) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB666 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB666_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB666(p)] : \
	            GP_PIXEL_GET_R_RGB666(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB666 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB666_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB666 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB666_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 255 : 63)

/**
 * @brief Returns a lookup table to encode RGB666 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB666_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB666 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB666_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB666(p)] : \
	            GP_PIXEL_GET_G_RGB666(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB666 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB666_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB666 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB666_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 255 : 63)

/**
 * @brief Returns a lookup table to encode RGB666 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB666_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB666 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB666_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB666(p)] : \
	            GP_PIXEL_GET_B_RGB666(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB666 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB666_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB666 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB666_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 255 : 63)

/**
 * @brief Returns a lookup table to encode RGB666 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB666_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a RGB332 pixel from linearized channel values.
 *
 * @param R A RGB332 linearized R channel value.
 * @param G A RGB332 linearized G channel value.
 * @param B A RGB332 linearized B channel value.
 * @param R_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param G_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param B_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A RGB332 pixel value.
 */
#define GP_PIXEL_CREATE_RGB332_ENC(R, G, B, R_gamma_enc, G_gamma_enc, B_gamma_enc) \
	(0\
	+ ((R_gamma_enc ? R_gamma_enc[(R)] : R) << 5) \
	+ ((G_gamma_enc ? G_gamma_enc[(G)] : G) << 2) \
	+ ((B_gamma_enc ? B_gamma_enc[(B)] : B) << 0) \
	)
/**
 * @brief Macro to get linearized R channel value for RGB332 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_R_RGB332_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_R_RGB332(p)] : \
	            GP_PIXEL_GET_R_RGB332(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB332 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB332_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB332 pixel R channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB332_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 31 : 7)

/**
 * @brief Returns a lookup table to encode RGB332 pixel R channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB332_R(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized G channel value for RGB332 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_G_RGB332_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_G_RGB332(p)] : \
	            GP_PIXEL_GET_G_RGB332(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB332 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB332_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB332 pixel G channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB332_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 31 : 7)

/**
 * @brief Returns a lookup table to encode RGB332 pixel G channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB332_G(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized B channel value for RGB332 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_B_RGB332_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_B_RGB332(p)] : \
	            GP_PIXEL_GET_B_RGB332(p)              \
)

/**
 * @brief Returns a lookup table to linearize RGB332 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_RGB332_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized RGB332 pixel B channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_RGB332_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 15 : 3)

/**
 * @brief Returns a lookup table to encode RGB332 pixel B channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_RGB332_B(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief A macro to compose a CMYK8888 pixel from linearized channel values.
 *
 * @param K A CMYK8888 linearized K channel value.
 * @param Y A CMYK8888 linearized Y channel value.
 * @param M A CMYK8888 linearized M channel value.
 * @param C A CMYK8888 linearized C channel value.
 * @param K_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param Y_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param M_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param C_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A CMYK8888 pixel value.
 */
#define GP_PIXEL_CREATE_CMYK8888_ENC(K, Y, M, C, K_gamma_enc, Y_gamma_enc, M_gamma_enc, C_gamma_enc) \
	(0\
	+ ((K_gamma_enc ? K_gamma_enc[(K)] : K) << 24) \
	+ ((Y_gamma_enc ? Y_gamma_enc[(Y)] : Y) << 16) \
	+ ((M_gamma_enc ? M_gamma_enc[(M)] : M) << 8) \
	+ ((C_gamma_enc ? C_gamma_enc[(C)] : C) << 0) \
	)
/**
 * @brief Macro to get linearized K channel value for CMYK8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_K_CMYK8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_K_CMYK8888(p)] : \
	            GP_PIXEL_GET_K_CMYK8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize CMYK8888 pixel K channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_CMYK8888_K(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized CMYK8888 pixel K channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_CMYK8888_K(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode CMYK8888 pixel K channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_CMYK8888_K(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized Y channel value for CMYK8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_Y_CMYK8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_Y_CMYK8888(p)] : \
	            GP_PIXEL_GET_Y_CMYK8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize CMYK8888 pixel Y channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_CMYK8888_Y(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized CMYK8888 pixel Y channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_CMYK8888_Y(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode CMYK8888 pixel Y channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_CMYK8888_Y(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief Macro to get linearized M channel value for CMYK8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_M_CMYK8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_M_CMYK8888(p)] : \
	            GP_PIXEL_GET_M_CMYK8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize CMYK8888 pixel M channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_CMYK8888_M(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? (pixmap)->gamma->lin[2]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized CMYK8888 pixel M channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_CMYK8888_M(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[2]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode CMYK8888 pixel M channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_CMYK8888_M(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[2]) ? (pixmap)->gamma->enc[2]->u8 : NULL)

/**
 * @brief Macro to get linearized C channel value for CMYK8888 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_C_CMYK8888_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_C_CMYK8888(p)] : \
	            GP_PIXEL_GET_C_CMYK8888(p)              \
)

/**
 * @brief Returns a lookup table to linearize CMYK8888 pixel C channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_CMYK8888_C(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[3]) ? (pixmap)->gamma->lin[3]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized CMYK8888 pixel C channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_CMYK8888_C(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[3]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode CMYK8888 pixel C channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_CMYK8888_C(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[3]) ? (pixmap)->gamma->enc[3]->u8 : NULL)

/**
 * @brief A macro to compose a G1_DB pixel from linearized channel values.
 *
 * @param V A G1_DB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G1_DB pixel value.
 */
#define GP_PIXEL_CREATE_G1_DB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G1_DB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G1_DB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G1_DB(p)] : \
	            GP_PIXEL_GET_V_G1_DB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G1_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G1_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G1_DB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G1_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1 : 1)

/**
 * @brief Returns a lookup table to encode G1_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G1_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G2_DB pixel from linearized channel values.
 *
 * @param V A G2_DB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G2_DB pixel value.
 */
#define GP_PIXEL_CREATE_G2_DB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G2_DB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G2_DB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G2_DB(p)] : \
	            GP_PIXEL_GET_V_G2_DB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G2_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G2_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G2_DB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G2_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 15 : 3)

/**
 * @brief Returns a lookup table to encode G2_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G2_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G4_DB pixel from linearized channel values.
 *
 * @param V A G4_DB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G4_DB pixel value.
 */
#define GP_PIXEL_CREATE_G4_DB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G4_DB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G4_DB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G4_DB(p)] : \
	            GP_PIXEL_GET_V_G4_DB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G4_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G4_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G4_DB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G4_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 63 : 15)

/**
 * @brief Returns a lookup table to encode G4_DB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G4_DB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G1_UB pixel from linearized channel values.
 *
 * @param V A G1_UB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G1_UB pixel value.
 */
#define GP_PIXEL_CREATE_G1_UB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G1_UB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G1_UB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G1_UB(p)] : \
	            GP_PIXEL_GET_V_G1_UB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G1_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G1_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G1_UB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G1_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1 : 1)

/**
 * @brief Returns a lookup table to encode G1_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G1_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G2_UB pixel from linearized channel values.
 *
 * @param V A G2_UB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G2_UB pixel value.
 */
#define GP_PIXEL_CREATE_G2_UB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G2_UB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G2_UB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G2_UB(p)] : \
	            GP_PIXEL_GET_V_G2_UB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G2_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G2_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G2_UB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G2_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 15 : 3)

/**
 * @brief Returns a lookup table to encode G2_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G2_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G4_UB pixel from linearized channel values.
 *
 * @param V A G4_UB linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G4_UB pixel value.
 */
#define GP_PIXEL_CREATE_G4_UB_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G4_UB pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G4_UB_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G4_UB(p)] : \
	            GP_PIXEL_GET_V_G4_UB(p)              \
)

/**
 * @brief Returns a lookup table to linearize G4_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G4_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized G4_UB pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G4_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 63 : 15)

/**
 * @brief Returns a lookup table to encode G4_UB pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G4_UB_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a G8 pixel from linearized channel values.
 *
 * @param V A G8 linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G8 pixel value.
 */
#define GP_PIXEL_CREATE_G8_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G8 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G8_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G8(p)] : \
	            GP_PIXEL_GET_V_G8(p)              \
)

/**
 * @brief Returns a lookup table to linearize G8 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G8_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized G8 pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G8_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode G8 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G8_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief A macro to compose a GA88 pixel from linearized channel values.
 *
 * @param V A GA88 linearized V channel value.
 * @param A A GA88 linearized A channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 * @param A_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A GA88 pixel value.
 */
#define GP_PIXEL_CREATE_GA88_ENC(V, A, V_gamma_enc, A_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	+ ((A_gamma_enc ? A_gamma_enc[(A)] : A) << 8) \
	)
/**
 * @brief Macro to get linearized V channel value for GA88 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_GA88_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_GA88(p)] : \
	            GP_PIXEL_GET_V_GA88(p)              \
)

/**
 * @brief Returns a lookup table to linearize GA88 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_GA88_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized GA88 pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_GA88_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 1023 : 255)

/**
 * @brief Returns a lookup table to encode GA88 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_GA88_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u8 : NULL)

/**
 * @brief Macro to get linearized A channel value for GA88 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_A_GA88_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_A_GA88(p)] : \
	            GP_PIXEL_GET_A_GA88(p)              \
)

/**
 * @brief Returns a lookup table to linearize GA88 pixel A channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_GA88_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? (pixmap)->gamma->lin[1]->u8 : NULL)

/**
 * @brief Returns a maximum value for a linearized GA88 pixel A channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_GA88_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[1]) ? 255 : 255)

/**
 * @brief Returns a lookup table to encode GA88 pixel A channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_GA88_A(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[1]) ? (pixmap)->gamma->enc[1]->u8 : NULL)

/**
 * @brief A macro to compose a G16 pixel from linearized channel values.
 *
 * @param V A G16 linearized V channel value.
 * @param V_gamma_enc A pointer to the #gp_gamma_table u8 or u16
 *        lookup table for the corresponding pixel channel.
 *
 * @return A G16 pixel value.
 */
#define GP_PIXEL_CREATE_G16_ENC(V, V_gamma_enc) \
	(0\
	+ ((V_gamma_enc ? V_gamma_enc[(V)] : V) << 0) \
	)
/**
 * @brief Macro to get linearized V channel value for G16 pixel.
 *
 * @param p A pixel value.
 * @param gamma_lin A pointer to the #gp_gamma_table u8 or u16 lookup table for
 *        the corresponding pixel channel.
 *
 * @return A linearized pixel channel value.
 */
#define GP_PIXEL_GET_V_G16_LIN(p, gamma_lin) (         \
	gamma_lin ? gamma_lin[GP_PIXEL_GET_V_G16(p)] : \
	            GP_PIXEL_GET_V_G16(p)              \
)

/**
 * @brief Returns a lookup table to linearize G16 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_LIN_TBL_G16_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? (pixmap)->gamma->lin[0]->u16 : NULL)

/**
 * @brief Returns a maximum value for a linearized G16 pixel V channel.
 *
 * The maximum value for a linearized pixel value depends on the #gp_pixmap #gp_gamma_table.
 *
 * @param pixmap A pixmap to get the maximum for.
 * @return A maximum value for the linearized pixel channel.
 */
#define GP_CHAN_LIN_MAX_VAL_G16_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->lin[0]) ? 262143 : 65535)

/**
 * @brief Returns a lookup table to encode G16 pixel V channel.
 *
 * @param pixmap A pixmap to get the table from.
 * @return A lookup table or NULL if channel is linear.
 */
#define GP_CHAN_TO_ENC_TBL_G16_V(pixmap) \
	(((pixmap)->gamma && (pixmap)->gamma->enc[0]) ? (pixmap)->gamma->enc[0]->u16 : NULL)

#endif /* GP_GAMMA_CORRECTION_GEN_H */
